import 'package:fis_measure/interfaces/process/items/item.dart';
import 'package:fis_measure/interfaces/process/items/item_metas.dart';
import 'package:fis_measure/process/calcuators/urm.dart';
import 'package:fis_measure/process/items/top_item_feature.dart';
import 'package:fis_measure/process/primitives/combos/urm_den_combo.dart';
import 'package:fis_measure/process/primitives/straightline.dart';
import 'package:fis_measure/process/primitives/urm_rect.dart';
import 'package:fis_measure/process/primitives/urm_straightline.dart';

class URMDen extends URMDenAbstract<URMVelFeature> {
  static const String _area1Key = "SRCurvature";
  static const String _area2Key = "SRRoiDensity";
  static const String _area3Key = "SRROIFractalDim";
  static const String _area4Key = "Vessel diameter";
  static const String _area5Key = "Vessel distance";

  late final URMStraightLine a1;
  late final URMRect a2;
  late final URMRect a3;
  late final StraightLine a4;
  late final StraightLine a5;

  URMDen(ItemMeta meta) : super(meta) {
    final metaA1 = meta.getChildByName(_area1Key)!;
    final metaA2 = meta.getChildByName(_area2Key)!;
    final metaA3 = meta.getChildByName(_area3Key)!;
    final metaA4 = meta.getChildByName(_area4Key)!;
    final metaA5 = meta.getChildByName(_area5Key)!;
    a1 = URMStraightLine.createURMStraightline(metaA1, this);
    a2 = URMRect.createMeasureRect(metaA2, this);
    a3 = URMRect.createMeasureRect(metaA3, this);
    a4 = StraightLine.createDistance(metaA4, this);
    a5 = StraightLine.createDistance(metaA5, this);
    childItems.add(a1);
    childItems.add(a2);
    childItems.add(a3);
    childItems.add(a4);
    childItems.add(a5);
  }

  @override
  StraightLine get child1 => a1;

  @override
  URMRect get child2 => a2;

  @override
  URMRect get child3 => a3;

  @override
  StraightLine get child4 => a4;

  @override
  StraightLine get child5 => a5;

  @override
  URMVelFeature buildFeature() => URMVelFeature(this);

  @override
  void onCancelingOnce() {}

  static URMDen crateURMDen(ItemMeta meta, [IMeasureItem? parent]) {
    var urmDen = URMDen(meta);
    urmDen.calculator = URMDenCal(urmDen);

    return urmDen;
  }
}

class URMVelFeature extends TopMeasureItemFeature {
  URMVelFeature(
    ITopMeasureItem refItem,
  ) : super(refItem);
}
