import 'package:fis_measure/interfaces/process/items/item.dart';
import 'package:fis_measure/interfaces/process/items/item_metas.dart';
import 'package:fis_measure/process/calcuators/urm.dart';
import 'package:fis_measure/process/items/top_item_feature.dart';
import 'package:fis_measure/process/primitives/combos/urm_vel_combo.dart';
import 'package:fis_measure/process/primitives/straightline.dart';
import 'package:fis_measure/process/primitives/urm_location.dart';

class URMVel extends URMVelAbstract<URMVelFeature> {
  static const String _area1Key = "SRLoactionVel";
  static const String _area2Key = "Vessel diameter";
  static const String _area3Key = "Vessel distance";

  late final URMLoaction a1;
  late final StraightLine a2;
  late final StraightLine a3;

  URMVel(ItemMeta meta) : super(meta) {
    final metaA1 = meta.getChildByName(_area1Key)!;
    final metaA2 = meta.getChildByName(_area2Key)!;
    final metaA3 = meta.getChildByName(_area3Key)!;
    a1 = URMLoaction.createURMLocation(metaA1, this);
    a2 = StraightLine.createDistance(metaA2, this);
    a3 = StraightLine.createDistance(metaA3, this);
    childItems.add(a1);
    childItems.add(a2);
    childItems.add(a3);
  }

  @override
  URMLoaction get child1 => a1;

  @override
  StraightLine get child2 => a2;

  @override
  StraightLine get child3 => a3;

  @override
  URMVelFeature buildFeature() => URMVelFeature(this);

  @override
  void onCancelingOnce() {}

  static URMVel crateURMVel(ItemMeta meta, [IMeasureItem? parent]) {
    var urmVel = URMVel(meta);
    urmVel.calculator = URMVelCal(urmVel);

    return urmVel;
  }
}

class URMVelFeature extends TopMeasureItemFeature {
  URMVelFeature(
    ITopMeasureItem refItem,
  ) : super(refItem);
}
