import 'package:fis_measure/interfaces/enums/items.dart';
import 'package:fis_measure/interfaces/process/items/item.dart';
import 'package:fis_measure/interfaces/process/items/item_metas.dart';
import 'package:fis_measure/interfaces/process/workspace/point_info.dart';
import 'package:fis_measure/process/calcuators/urm.dart';
import 'package:fis_measure/process/primitives/straightline.dart';
import 'package:fis_measure/utils/prompt_box.dart';

class URMStraightLine extends StraightLine {
  URMStraightLine(ItemMeta meta, IMeasureItem? parent) : super(meta, parent);

  static URMStraightLine createURMStraightline(ItemMeta meta,
      [IMeasureItem? parent]) {
    URMStraightLine sraightLine = URMStraightLine(meta, parent);
    sraightLine.repeatableEditable = true;
    sraightLine.calculator = URMStraightLineCal(sraightLine, type: meta.name);
    return sraightLine;
  }

  @override
  bool onExecuteMouse(PointInfo args) {
    if (waitingResult) return false;
    if (state == ItemStates.finished) {
      if (args.pointType == PointInfoType.mouseDown) {
        state = ItemStates.waiting;
      } else {
        return false;
      }
    }

    if (state == ItemStates.waiting) {
      if (args.pointType == PointInfoType.mouseDown) {
        handleMouseDownWhileWaiting(args);
      }
    } else if (state == ItemStates.running) {
      if (args.pointType == PointInfoType.mouseUp) return false;

      feature?.endPoint = args;
      if (args.pointType == PointInfoType.mouseDown) {
        handleFinish();

        /// 重置十字样式
        // changeCrossIndicatorStyle(CrossIndicatorStyle.nomal);
      }
    }
    return true;
  }

  bool waitingResult = false;

  void handleFinish() async {
    feature!.isActive = false;
    PromptBox.loading("分析中...");
    waitingResult = true;
    await doCalculateAsync();
    doFeatureFinish();
    PromptBox.dismiss();
    waitingResult = false;
  }
}
