// ignore_for_file: must_be_immutable

import 'package:fis_i18n/i18n.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/define.dart';
import 'package:fis_ui/index.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:sleek_circular_slider/sleek_circular_slider.dart';
import 'package:fis_theme/theme.dart';
import '../controller.dart';

/// AI诊断结果
class AiDiagnosticResult extends FisView<AiResultModifierController> {
  final AIDetectedObject aiDetectedObject;

  final DiagnosisOrganEnum diagnosisOrgan;
  const AiDiagnosticResult(
    this.aiDetectedObject,
    this.diagnosisOrgan, {
    Key? key,
  }) : super(key: key);

  @override
  FWidget build(BuildContext context) {
    return FContainer(
      padding: const EdgeInsets.only(bottom: 10),
      child: FColumn(
        children: [
          FRow(
            mainAxisSize: MainAxisSize.max,
            children: [
              FExpanded(
                child: FContainer(
                  decoration: const BoxDecoration(
                    borderRadius: BorderRadius.only(
                      topLeft: Radius.circular(4),
                      topRight: Radius.circular(4),
                    ),
                    color: Color.fromRGBO(54, 169, 206, 1),
                  ),
                  padding: const EdgeInsets.only(
                    top: 4,
                    bottom: 10,
                    left: 8,
                    right: 8,
                  ),
                  child: FText(
                    i18nBook.measure.aiDiagnosticResults.t,
                    style: const TextStyle(
                      color: Colors.white,
                    ),
                  ),
                ),
              )
            ],
          ),
          FContainer(
            decoration: BoxDecoration(
              border: Border.all(
                color: const Color.fromRGBO(54, 169, 206, 1),
              ),
              color: Colors.transparent,
            ),
            padding: const EdgeInsets.only(bottom: 5),
            child: _buildPossibilityProgressBar(),
          ),
        ],
      ),
    );
  }

  FWidget _aiResultValue() {
    List<FSelectOptionModel> source = _initEnumFieldList();
    return FContainer(
      margin: const EdgeInsets.only(top: 10),
      child: FSelect<FSelectOptionModel, int>(
        height: 38,
        width: 260,
        source: source,
        hintText: i18nBook.common.Select.t,
        fontFamily: FTheme.ins.localeSetting.fontFamily,
        optionValueExtractor: (data) => data.value,
        optionLabelExtractor: (data) => data.title,
        value: aiDetectedObject.label,
        textColor: Colors.white,
        onSelectChanged: (value, index) {
          aiDetectedObject.label = index!;
        },
      ),
    );
  }

  String _buildAITitle() {
    switch (diagnosisOrgan) {
      case DiagnosisOrganEnum.Breast:
        return 'Breast';
      case DiagnosisOrganEnum.Liver:
        return 'Liver';
      case DiagnosisOrganEnum.Thyroid:
        return 'Thyroid';
      default:
        return '';
    }
  }

  FWidget _buildLesionSize() {
    return FisBuilder<AiResultModifierController>(
        id: 'ai_result_lesion_size',
        builder: (_) {
          return FContainer(
            margin: const EdgeInsets.symmetric(horizontal: 3, vertical: 2),
            child: FText(
              "${_.horizontalLength} x ${_.verticalLength}",
              style: const TextStyle(color: Colors.white),
            ),
          );
        });
  }

  FWidget _buildLesionRatio() {
    return FisBuilder<AiResultModifierController>(
      id: 'ai_result_lesion_ratio',
      builder: (_) {
        return FContainer(
          margin: const EdgeInsets.symmetric(horizontal: 3, vertical: 2),
          child: FText(
            _.lesionRatio,
            style: const TextStyle(color: Colors.white),
          ),
        );
      },
    );
  }

  FWidget _buildPossibilityProgressBar() {
    return FRow(
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: [
        FExpanded(
          child: FContainer(
            padding: const EdgeInsets.only(
              left: 10,
            ),
            child: FColumn(
              crossAxisAlignment: CrossAxisAlignment.start,
              mainAxisAlignment: MainAxisAlignment.start,
              children: [
                _buildTitle(
                  i18nBook.measure.analysisConclusion.t,
                  _aiResultValue(),
                ),
                const FSizedBox(
                  height: 15,
                ),
                FRow(
                  children: [
                    _buildTitle(
                      i18nBook.measure.isLesionSize.t,
                      _buildLesionSize(),
                    ),
                    const FSizedBox(
                      width: 20,
                    ),
                    _buildTitle(
                      i18nBook.measure.lesionSize.t,
                      _buildLesionRatio(),
                    ),
                  ],
                )
              ],
            ),
          ),
        ),
      ],
    );
  }

  FWidget _buildTitle(String label, FWidget value) {
    return FColumn(
      mainAxisSize: MainAxisSize.max,
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        FText(
          label,
          style: const TextStyle(
            color: Color.fromRGBO(54, 169, 206, 1),
          ),
        ),
        // const FSizedBox(
        //   height: 10,
        // ),
        value,
        // const FSizedBox(
        //   height: 5,
        // ),
      ],
    );
  }

  List<FSelectOptionModel> _initEnumFieldList() {
    List<FSelectOptionModel> source = [];
    if (controller.diagnosisEnumItems.isNotEmpty) {
      List<EnumFieldDTO> enumFieldList =
          controller.diagnosisEnumItems.firstWhereOrNull((element) {
                return element.code == _buildAITitle();
              })?.children ??
              [];
      for (var element in enumFieldList) {
        source.add(
          FSelectOptionModel(
            title: controller.getValuesFromAiLanguage(element.value ?? ''),
            value: element.id,
          ),
        );
      }
    }
    return source;
  }
}
